@@ -4,7 +4,6 @@
 #include "detection/gpu/gpu_driver_specific.h"
 #include "common/io/io.h"
 #include "common/library.h"
-#include "common/properties.h"
 #include "util/stringUtils.h"
 #include "util/mallocHelper.h"
 
@@ -22,10 +21,16 @@
     #include <sys/ioctl.h>
 #endif
 
-#include "gpu_asahi.h"
-
-#define FF_STR_INDIR(x) #x
-#define FF_STR(x) FF_STR_INDIR(x)
+#if defined(FF_HAVE_DRM) && defined(__aarch64__)
+    // https://github.com/alyssarosenzweig/linux/blob/agx-uapi-v7/include/uapi/drm/asahi_drm.h
+    // Found in kernel-headers-6.14.4-400.asahi.fc42.aarch64
+    #if __has_include(<drm/asahi_drm.h>)
+        #include <drm/asahi_drm.h>
+    #else
+        #include "asahi_drm.h"
+    #endif
+    #define FF_HAVE_DRM_ASAHI 1
+#endif
 
 static bool pciDetectDriver(FFstrbuf* result, FFstrbuf* pciDir, FFstrbuf* buffer, FF_MAYBE_UNUSED const char* drmKey)
 {
@@ -141,6 +146,26 @@ static const char* drmDetectAmdSpecific(const FFGPUOptions* options, FFGPUResult
         gpu->frequency = (uint32_t) (gpuInfo.max_engine_clk / 1000u);
         gpu->index = FF_GPU_INDEX_UNSET;
         gpu->type = gpuInfo.ids_flags & AMDGPU_IDS_FLAGS_FUSION ? FF_GPU_TYPE_INTEGRATED : FF_GPU_TYPE_DISCRETE;
+        #define FF_VRAM_CASE(name, value) case value /* AMDGPU_VRAM_TYPE_ ## name */: ffStrbufSetStatic(&gpu->memoryType, #name); break
+        switch (gpuInfo.vram_type)
+        {
+            FF_VRAM_CASE(UNKNOWN, 0);
+            FF_VRAM_CASE(GDDR1, 1);
+            FF_VRAM_CASE(DDR2, 2);
+            FF_VRAM_CASE(GDDR3, 3);
+            FF_VRAM_CASE(GDDR4, 4);
+            FF_VRAM_CASE(GDDR5, 5);
+            FF_VRAM_CASE(HBM, 6);
+            FF_VRAM_CASE(DDR3, 7);
+            FF_VRAM_CASE(DDR4, 8);
+            FF_VRAM_CASE(GDDR6, 9);
+            FF_VRAM_CASE(DDR5, 10);
+            FF_VRAM_CASE(LPDDR4, 11);
+            FF_VRAM_CASE(LPDDR5, 12);
+            default:
+                ffStrbufAppendF(&gpu->memoryType, "Unknown (%u)", gpuInfo.vram_type);
+                break;
+        }
 
         struct amdgpu_heap_info heapInfo;
         if (ffamdgpu_query_heap_info(handle, AMDGPU_GEM_DOMAIN_VRAM, 0, &heapInfo) >= 0)
@@ -226,10 +251,15 @@ static void pciDetectAmdSpecific(const FFGPUOptions* options, FFGPUResult* gpu,
                     gpu->shared.used = value;
             }
         }
+
+        ffStrbufSubstrBefore(pciDir, pciDirLen);
+        ffStrbufAppendS(pciDir, "/gpu_busy_percent");
+        if (ffReadFileBuffer(pciDir->chars, buffer) && (value = ffStrbufToUInt(buffer, 0)))
+            gpu->coreUsage = (double) value;
     }
 }
 
-static void pciDetectIntelSpecific(FFGPUResult* gpu, FFstrbuf* pciDir, FFstrbuf* buffer, const char* drmKey)
+static void pciDetectIntelSpecific(const FFGPUOptions* options, FFGPUResult* gpu, FFstrbuf* pciDir, FFstrbuf* buffer, const char* drmKey)
 {
     // Works for Intel GPUs
     // https://patchwork.kernel.org/project/intel-gfx/patch/1422039866-11572-3-git-send-email-ville.syrjala@linux.intel.com/
@@ -239,12 +269,46 @@ static void pciDetectIntelSpecific(FFGPUResult* gpu, FFstrbuf* pciDir, FFstrbuf*
 
     if (!drmKey) return;
 
-    if (ffStrbufEqualS(&gpu->driver, "xe"))
+    const uint32_t pciDirLen = pciDir->length;
+
+    bool isXE = ffStrbufEqualS(&gpu->driver, "xe");
+    if (isXE)
         ffStrbufAppendS(pciDir, "/tile0/gt0/freq0/max_freq");
     else
         ffStrbufAppendF(pciDir, "/drm/%s/gt_max_freq_mhz", drmKey);
     if (ffReadFileBuffer(pciDir->chars, buffer))
         gpu->frequency = (uint32_t) ffStrbufToUInt(buffer, 0);
+    ffStrbufSubstrBefore(pciDir, pciDirLen);
+
+    if (options->temp)
+    {
+        ffStrbufAppendS(pciDir, "/hwmon/");
+        FF_AUTO_CLOSE_DIR DIR* dirp = opendir(pciDir->chars);
+        if (dirp)
+        {
+            struct dirent* entry;
+            while ((entry = readdir(dirp)) != NULL)
+            {
+                if (entry->d_name[0] == '.') continue;
+
+                ffStrbufSubstrBefore(pciDir, pciDirLen + strlen("/hwmon/"));
+                ffStrbufAppendS(pciDir, entry->d_name);
+                // https://github.com/Syllo/nvtop/blob/73291884d926445e499d6b9b71cb7a9bdbc7c393/src/extract_gpuinfo_intel.c#L279-L281
+                ffStrbufAppendS(pciDir, isXE ? "/temp2_input" : "/temp1_input");
+
+                if (ffReadFileBuffer(pciDir->chars, buffer))
+                {
+                    uint64_t value = ffStrbufToUInt(buffer, 0);
+                    if (value > 0)
+                    {
+                        gpu->temperature = (double) value / 1000;
+                        break;
+                    }
+                }
+            }
+        }
+        ffStrbufSubstrBefore(pciDir, pciDirLen);
+    }
 }
 
 static inline int popcountBytes(uint8_t* bytes, uint32_t length)
@@ -427,7 +491,9 @@ static const char* detectPci(const FFGPUOptions* options, FFlist* gpus, FFstrbuf
     }
     else
     {
-        pPciPath = memrchr(deviceDir->chars, '/', deviceDir->length) + 1;
+        pPciPath = memrchr(deviceDir->chars, '/', deviceDir->length);
+        assert(pPciPath);
+        pPciPath++;
     }
 
     uint32_t pciDomain, pciBus, pciDevice, pciFunc;
@@ -439,6 +505,7 @@ static const char* detectPci(const FFGPUOptions* options, FFlist* gpus, FFstrbuf
     ffStrbufInit(&gpu->name);
     ffStrbufInit(&gpu->driver);
     ffStrbufInit(&gpu->platformApi);
+    ffStrbufInit(&gpu->memoryType);
     gpu->index = FF_GPU_INDEX_UNSET;
     gpu->temperature = FF_GPU_TEMP_UNSET;
     gpu->coreUsage = FF_GPU_CORE_USAGE_UNSET;
@@ -491,22 +558,14 @@ static const char* detectPci(const FFGPUOptions* options, FFlist* gpus, FFstrbuf
                 char* pend;
                 uint64_t revision = strtoul(buffer->chars, &pend, 16);
                 if (pend != buffer->chars)
-                {
-                    char query[32];
-                    snprintf(query, ARRAY_SIZE(query), "%X,\t%X,", (unsigned) deviceId, (unsigned) revision);
-                    #ifdef FF_CUSTOM_AMDGPU_IDS_PATH
-                    ffParsePropFile(FF_STR(FF_CUSTOM_AMDGPU_IDS_PATH), query, &gpu->name);
-                    #else
-                    ffParsePropFileData("libdrm/amdgpu.ids", query, &gpu->name);
-                    #endif
-                }
+                    ffGPUQueryAmdGpuName((uint16_t) deviceId, (uint8_t) revision, gpu);
             }
             ffStrbufSubstrBefore(deviceDir, drmDirPathLength);
         }
     }
     else if (gpu->vendor.chars == FF_GPU_VENDOR_NAME_INTEL)
     {
-        pciDetectIntelSpecific(gpu, deviceDir, buffer, drmKey);
+        pciDetectIntelSpecific(options, gpu, deviceDir, buffer, drmKey);
         ffStrbufSubstrBefore(deviceDir, drmDirPathLength);
         if (options->driverSpecific && drmKey)
             drmDetectIntelSpecific(gpu, drmKey, buffer);
@@ -562,13 +621,13 @@ static const char* detectPci(const FFGPUOptions* options, FFlist* gpus, FFstrbuf
 
 #if __aarch64__
 
-FF_MAYBE_UNUSED static const char* drmDetectAsahiSpecific(FFGPUResult* gpu, const char* name, FFstrbuf* buffer, const char* drmKey)
+FF_MAYBE_UNUSED static const char* drmDetectAsahiSpecific(FFGPUResult* gpu, const char* name, FF_MAYBE_UNUSED FFstrbuf* buffer, FF_MAYBE_UNUSED const char* drmKey)
 {
     if (sscanf(name, "agx-t%lu", &gpu->deviceId) == 1)
         ffStrbufSetStatic(&gpu->name, ffCPUAppleCodeToName((uint32_t) gpu->deviceId));
     ffStrbufSetStatic(&gpu->vendor, FF_GPU_VENDOR_NAME_APPLE);
 
-    #if FF_HAVE_DRM
+    #if FF_HAVE_DRM_ASAHI
     ffStrbufSetS(buffer, "/dev/dri/");
     ffStrbufAppendS(buffer, drmKey);
     FF_AUTO_CLOSE_FD int fd = open(buffer->chars, O_RDONLY);
@@ -581,38 +640,32 @@ FF_MAYBE_UNUSED static const char* drmDetectAsahiSpecific(FFGPUResult* gpu, cons
             .size = sizeof(paramsGlobal),
         }) >= 0)
         {
-            ffStrbufAppendF(&gpu->driver, " %u", paramsGlobal.unstable_uabi_version);
+            // They removed `unstable_uabi_version` from the struct. Hopefully they won't introduce new ABI changes.
+            gpu->coreCount = (int32_t) (paramsGlobal.num_clusters_total * paramsGlobal.num_cores_per_cluster);
+            gpu->frequency = paramsGlobal.max_frequency_khz / 1000;
+            gpu->deviceId = paramsGlobal.chip_id;
 
-            // FIXME: They will introduce ABI breaking changes. Always check the latest version
-            // https://www.reddit.com/r/AsahiLinux/comments/1ei2qiv/comment/lgm0v5s/
-            if (paramsGlobal.unstable_uabi_version == DRM_ASAHI_UNSTABLE_UABI_VERSION)
+            if (!gpu->name.length)
             {
-                gpu->coreCount = (int) paramsGlobal.num_cores_total_active;
-                gpu->frequency = paramsGlobal.max_frequency_khz / 1000;
-                gpu->deviceId = paramsGlobal.chip_id;
-
-                if (!gpu->name.length)
-                {
-                    const char* variant = " Unknown";
-                    switch (paramsGlobal.gpu_variant) {
-                    case 'G':
-                        variant = "";
-                        break;
-                    case 'S':
-                        variant = " Pro";
-                        break;
-                    case 'C':
-                        variant = " Max";
-                        break;
-                    case 'D':
-                        variant = " Ultra";
-                        break;
-                    }
-                    ffStrbufSetF(&gpu->name, "Apple M%d%s (G%d%c %02X)",
-                        paramsGlobal.gpu_generation - 12, variant,
-                        paramsGlobal.gpu_generation, paramsGlobal.gpu_variant,
-                        paramsGlobal.gpu_revision + 0xA0);
+                const char* variant = " Unknown";
+                switch (paramsGlobal.gpu_variant) {
+                case 'G':
+                    variant = "";
+                    break;
+                case 'S':
+                    variant = " Pro";
+                    break;
+                case 'C':
+                    variant = " Max";
+                    break;
+                case 'D':
+                    variant = " Ultra";
+                    break;
                 }
+                ffStrbufSetF(&gpu->name, "Apple M%d%s (G%d%c %02X)",
+                    paramsGlobal.gpu_generation - 12, variant,
+                    paramsGlobal.gpu_generation, paramsGlobal.gpu_variant,
+                    paramsGlobal.gpu_revision + 0xA0);
             }
         }
     }
@@ -641,6 +694,7 @@ static const char* detectOf(FFlist* gpus, FFstrbuf* buffer, FFstrbuf* drmDir, co
     ffStrbufInit(&gpu->name);
     ffStrbufInit(&gpu->vendor);
     ffStrbufInit(&gpu->driver);
+    ffStrbufInit(&gpu->memoryType);
     ffStrbufInitF(&gpu->platformApi, "DRM (%s)", drmKey);
     gpu->temperature = FF_GPU_TEMP_UNSET;
     gpu->coreCount = FF_GPU_CORE_COUNT_UNSET;
