@@ -34,6 +34,7 @@
 #include "modules/icons/option.h"
 #include "modules/initsystem/option.h"
 #include "modules/kernel/option.h"
+#include "modules/keyboard/option.h"
 #include "modules/loadavg/option.h"
 #include "modules/locale/option.h"
 #include "modules/lm/option.h"
@@ -41,6 +42,7 @@
 #include "modules/media/option.h"
 #include "modules/memory/option.h"
 #include "modules/monitor/option.h"
+#include "modules/mouse/option.h"
 #include "modules/netio/option.h"
 #include "modules/opengl/option.h"
 #include "modules/opencl/option.h"
